<?php
class Cx_Optimization {

    // 功能控制
    public $optiondata;

    // 加载类项目
    public static function action(){
		global $Cx_Optimization;
        $Cx_Optimization = new self();
        $Cx_Optimization->construct();
        $Cx_Optimization->option_data();
        $Cx_Optimization->begin();        	
    }

    public function construct(){
        // 移除顶部工具栏
        add_filter('show_admin_bar',          '__return_false');
        // 修改wp头像调用规则
        add_filter('pre_get_avatar',             [&$this,'cxtheme_get_avatar'],10,3);
        // 上传支持svg
        add_filter( 'upload_mimes',              [&$this,'upload_images_svg'] );
        add_filter( 'max_srcset_image_width',    [&$this,'srcet_imagewidth'] );
    }

    
    
    public function option_data(){
        $option_data = [];
        if( cx_switcher('optimize-json') )      $option_data[] = 'wp-json';
        if( cx_switcher('optimize-emoji') )     $option_data[] = 'emoji';
        if( cx_switcher('optimize-head') )      $option_data[] = 'headtag';
        if( cx_switcher('optimize-embeds') )    $option_data[] = 'embeds';
        if( cx_switcher('optimize-google') )    $option_data[] = 'google';
        if( !function_exists('no_category_base_refresh_rules') && 
            cx_switcher('basic-nocategoty') )   $option_data[] = 'nocategoty';
        if(cx_switcher('optimize-prefetch') )   $option_data[] = 'prefetch';
        if(cx_switcher('optimize-headsvg') )    $option_data[] = 'headsvg';
        if(cx_switcher('optimize-styles') )     $option_data[] = 'globalstyles';
        $option_data[] = 'filter_menu';

        $this->optiondata = apply_filters('cxtheme-optimization-data',(array)$option_data);
    }    


    // 类核心
	function begin(){
        
        if( empty($this->optiondata) || !is_array($this->optiondata) ) return;
        foreach ($this->optiondata as $itemval) {
            switch ($itemval) {
            //移除wp-json链接
            case 'wp-json':
                add_filter('rest_enabled',            '__return_false');
                add_filter('rest_jsonp_enabled',      '__return_false');
                remove_action( 'wp_head',             'rest_output_link_wp_head', 10 );
                remove_action( 'wp_head',             'wp_oembed_add_discovery_links', 10 );
            //删除emoji脚本
            break;case 'emoji':
                remove_action( 'admin_print_scripts', 'print_emoji_detection_script');
                remove_action( 'admin_print_styles',  'print_emoji_styles');
                remove_action( 'wp_head',             'print_emoji_detection_script', 7);
                remove_action( 'wp_print_styles',     'print_emoji_styles');
                remove_filter( 'the_content_feed',    'wp_staticize_emoji');
                remove_filter( 'comment_text_rss',    'wp_staticize_emoji');
                remove_filter( 'wp_mail',             'wp_staticize_emoji_for_email');
            //移除头部冗余代码
            break;case 'headtag':
                remove_action( 'wp_head',             'wp_generator' );// WP版本信息
                remove_action( 'wp_head',             'rsd_link' );// 离线编辑器接口
                remove_action( 'wp_head',             'wlwmanifest_link' );// 同上
                //remove_action( 'wp_head',             'rel_canonical' );
                remove_action( 'wp_head',             'adjacent_posts_rel_link_wp_head', 10, 0 );// 上下文章的url
                remove_action( 'wp_head',             'feed_links', 2 );// 文章和评论feed
                remove_action( 'wp_head',             'feed_links_extra', 3 );// 去除评论feed
                remove_action( 'wp_head',             'wp_shortlink_wp_head', 10, 0 );// 短链接
            //禁用embeds功能
            break;case 'embeds':
                add_action( 'init',                   [&$this,'disable_embeds_init'], 9999 );
                register_activation_hook( __FILE__,   [&$this,'disable_embeds_remove_rewrite_rules'] );
                register_deactivation_hook( __FILE__, [&$this,'disable_embeds_flush_rewrite_rules'] );
            // 禁用谷歌字体
            break;case 'google':  
                add_action( 'init',                   [&$this,'wp_remove_open_sans_from_wp_core'] );
            // 分类去除category/前缀
            break;case 'nocategoty':
                register_activation_hook(__FILE__,   [&$this,'no_category_base_refresh_rules']);
                register_deactivation_hook(__FILE__, [&$this,'no_category_base_deactivate']);
                add_action('created_category',       [&$this,'no_category_base_refresh_rules']);
                add_action('delete_category',        [&$this,'no_category_base_refresh_rules']);
                add_action('edited_category',        [&$this,'no_category_base_refresh_rules']);
                add_action('init',                   [&$this,'no_category_base_permastruct']);
                add_filter('category_rewrite_rules', [&$this,'no_category_base_rewrite_rules']);
                add_filter('query_vars',             [&$this,'no_category_base_query_vars']);
                add_filter('request',                [&$this,'no_category_base_request']);
            //移除 Head 中的 dns-prefetch 标签
            break;case 'nocategoty':
                add_action( 'wp_resource_hints',     [&$this,'remove_dns_prefetch'], 10, 2 );
            // 禁止加载 global-styles-inline-css
            break;case 'globalstyles':
                remove_action('wp_enqueue_scripts', 'wp_enqueue_global_styles');
            // 禁止前台加载SVG代码
            break;case 'headsvg':
                remove_action( 'wp_body_open', 'wp_global_styles_render_svg_filters' );        
            // 过滤菜单    
            break;case 'filter_menu':
                add_filter('nav_menu_item_id', '__return_null', 10, 4);
                add_filter('nav_menu_css_class',  [&$this,'filter_menu_classes'], 10, 4);
                add_filter('nav_menu_item_title', [&$this,'filter_menu_item_title'], 10, 4);
            break;
            }
        }
    }
    /** 处理菜单 */
    function filter_menu_classes($classes, $item, $args, $depth) {
        $new_classes = [
            'menu-item'
        ];
        if( $item->current ){
            $new_classes[] = 'current';
        }
        if( in_array('current-post-ancestor',$classes) ){
            $new_classes[] = 'current-post-ancestor';
        }
        if( in_array('current-menu-parent',$classes) ){
            $new_classes[] = 'current-menu-parent';
        }
        if( in_array('menu-item-has-children',$classes) ){
            $new_classes[] = 'menu-has-children';
        }
        return $new_classes;
    }

    function filter_menu_item_title($title, $item, $args, $depth) {
        if(is_array($item->classes) && in_array('menu-item-has-children',$item->classes) ){
            $title .= ' <i class="cxicon cxicon-chevron-down menu-children-icon"></i>';
        }
        return $title;
    }


    function srcet_imagewidth(){
        return 1;
    }

    function upload_images_svg( $mimes ){
        $mimes['svg'] = 'image/svg+xml';
        return $mimes;
    }
    

    function cxtheme_get_avatar($avatar,$id_or_email, $args){
        if(is_object($id_or_email)){
            if( isset($id_or_email->ID) ){
                $id_or_email = $id_or_email->ID;
            }elseif( $id_or_email->comment_author_email ){
                $id_or_email = $id_or_email->comment_author_email;
                if( is_email($id_or_email) ){
                    $user = get_user_by('email', $id_or_email);
                    if( !empty($user) ){
                        $id_or_email = $user->ID;
                    }
                }
            }	        
        }

        if(is_numeric($id_or_email)){
            $update_avatar  = false;
            $new_avatar_url = '';
            $avatar_url = get_user_meta( $id_or_email, 'user_avatar', true );
            if( !empty($avatar_url) ){
                $avatar_url = json_decode($avatar_url,true);
            }else{
                $avatar_url = [];
            }
            if( !empty($avatar_url) && is_array($avatar_url) && isset($avatar_url['avatar']) ){
                if( isset($avatar_url['type']) && isset($avatar_url['avatar'][$avatar_url['type']]) ){
                    $new_avatar_url = $avatar_url['avatar'][$avatar_url['type']];
                }else{
                    if( isset($avatar_url['avatar']['custom']) ){
                        $new_avatar_url     = $avatar_url['avatar']['custom'];
                        $avatar_url['type'] = 'custom';
                        $update_avatar      = true;
                    }else{
                        foreach ($avatar_url['avatar'] as $key => $value) {
                            $new_avatar_url     = $value;
                            $update_avatar      = true;
                            $avatar_url['type'] = $key;
                            break;
                        }
                    }
                }
            }

            // 默认头像
            if( !$new_avatar_url ){
                $new_avatar_url = cx_option('user-avatar');
            }

            if( $new_avatar_url ){
                // 更新头像
                if( $update_avatar ){
                    update_user_meta($id_or_email, 'user_avatar', json_encode($avatar_url) );
                }
                // 输出Html
                $extra_attr = $args['extra_attr'];
                $loading    = $args['loading'];

                if ( in_array( $loading, array( 'lazy', 'eager' ), true ) && ! preg_match( '/\bloading\s*=/', $extra_attr ) ) {
                    if ( ! empty( $extra_attr ) ) {
                        $extra_attr .= ' ';
                    }

                    $extra_attr .= "loading='{$loading}'";
                }
                $avatar = sprintf(
                    "<img alt='%s' src='%s' class='%s' height='%d' width='%d' %s/>",
                    esc_attr( $args['alt'] ),
                    esc_url( $new_avatar_url ),
                    esc_attr( 'avatar' ),
                    (int) $args['height'],
                    (int) $args['width'],
                    $extra_attr
                );

            }   
        }
        return $avatar;
    }

    function remove_dns_prefetch( $hints, $relation_type ) {
        if ( 'dns-prefetch' === $relation_type ) {
        return array_diff( wp_dependencies_unique_hosts(), $hints );
        }
        return $hints;
    }

    function no_category_base_refresh_rules(){
        global $wp_rewrite;
        $wp_rewrite->flush_rules();
    }

    function no_category_base_deactivate(){
        remove_filter('category_rewrite_rules', [$this,'no_category_base_rewrite_rules']); // We don't want to insert our custom rules again
        $this->no_category_base_refresh_rules();
    }

    function no_category_base_permastruct(){
        global $wp_rewrite;
        global $wp_version;
        if ($wp_version >= 3.4) {
            $wp_rewrite->extra_permastructs['category']['struct'] = '%category%';
        } else {
            $wp_rewrite->extra_permastructs['category'][0] = '%category%';
        }
    }

    function no_category_base_rewrite_rules($category_rewrite){
        global $wp_rewrite;
        $category_rewrite = array();
        if (class_exists('Sitepress')) {
            global $sitepress;
            remove_filter('terms_clauses', array($sitepress, 'terms_clauses'));
            $categories = get_categories(array('hide_empty' => false));
            add_filter('terms_clauses', array($sitepress, 'terms_clauses'));
        } else {
            $categories = get_categories(array('hide_empty' => false));
        }

        foreach ($categories as $category) {
            $category_nicename = $category->slug;

            if ($category->parent == $category->cat_ID) {
                $category->parent = 0;
            } elseif ($category->parent != 0) {
                $category_nicename = get_category_parents($category->parent, false, '/', true) . $category_nicename;
            }

            $category_rewrite['(' . $category_nicename . ')/(?:feed/)?(feed|rdf|rss|rss2|atom)/?$']    = 'index.php?category_name=$matches[1]&feed=$matches[2]';
            $category_rewrite["({$category_nicename})/{$wp_rewrite->pagination_base}/?([0-9]{1,})/?$"] = 'index.php?category_name=$matches[1]&paged=$matches[2]';
            $category_rewrite['(' . $category_nicename . ')/?$']                                       = 'index.php?category_name=$matches[1]';
        }

        // Redirect support from Old Category Base
        $old_category_base                               = get_option('category_base') ? get_option('category_base') : 'category';
        $old_category_base                               = trim($old_category_base, '/');
        $category_rewrite[$old_category_base . '/(.*)$'] = 'index.php?category_redirect=$matches[1]';
        return $category_rewrite;
    }

    function no_category_base_query_vars($public_query_vars){
        $public_query_vars[] = 'category_redirect';
        return $public_query_vars;
    }

    function no_category_base_request($query_vars){
        if (isset($query_vars['category_redirect'])) {
            $catlink = trailingslashit(get_option('home')) . user_trailingslashit($query_vars['category_redirect'], 'category');
            status_header(301);
            header("Location: $catlink");
            exit();
        }

        return $query_vars;
    }


    // 移除 block-library CSS
    function remove_block_library_css() {
        wp_dequeue_style( 'wp-block-library' );
    }

    // 禁用谷歌字体
    function wp_remove_open_sans_from_wp_core() {
        wp_deregister_style( 'open-sans' );
        wp_register_style( 'open-sans', false );
        wp_enqueue_style('open-sans','');

        wp_deregister_style( 'wp-editor-font' );
        wp_register_style( 'wp-editor-font', false );
        wp_enqueue_style('wp-editor-font','');

    }

    //禁用embeds功能
	function disable_embeds_init() {
        global $wp;
        $wp->public_query_vars = array_diff( $wp->public_query_vars, array(
            'embed',
        ) );
        remove_action( 'rest_api_init',      'wp_oembed_register_route' );
        add_filter( 'embed_oembed_discover', '__return_false' );
        remove_filter( 'oembed_dataparse',   'wp_filter_oembed_result', 10 );
        remove_action( 'wp_head',            'wp_oembed_add_discovery_links' );
        remove_action( 'wp_head',            'wp_oembed_add_host_js' );
        add_filter( 'tiny_mce_plugins',      array($this,'disable_embeds_tiny_mce_plugin') );
        add_filter( 'rewrite_rules_array',   array($this,'disable_embeds_rewrites') );
	}

	function disable_embeds_tiny_mce_plugin( $plugins ) {
        return array_diff( $plugins, array( 'wpembed' ) );
	}
	function disable_embeds_rewrites( $rules ) {
        foreach ( $rules as $rule => $rewrite ) {
            if ( false !== strpos( $rewrite, 'embed=true' ) ) {
                unset( $rules[ $rule ] );
            }
        } 
        return $rules;
	}
	function disable_embeds_remove_rewrite_rules() {
        add_filter( 'rewrite_rules_array', array($this,'disable_embeds_rewrites') );
        flush_rewrite_rules();
	}    
}
// 加载类
Cx_Optimization::action();