<?php
class Cx_SetupTheme {

    // 加载类项目
    public static function action(){
		global $Cx_SetupTheme;
        $Cx_SetupTheme = new self();
		$Cx_SetupTheme->begin();	
    }

    // 类核心
	public function begin(){
        add_action( 'after_setup_theme',    	     [&$this, 'themes_setup'] );  // 初始化主题
        if( cx_switcher('seo-off') ){
            // SEO 相关配置
            add_filter( 'document_title_separator',  [&$this, 'title_separator'] );
            add_filter( 'document_title_parts',      [&$this, 'title_parts'] );
            add_action( 'wp_head', 					 [&$this, 'keywords_desc_web'],1);
        }

        // 加载脚本文件
        add_action( 'wp_enqueue_scripts',            [&$this,'add_scripts'] );
        add_filter( 'body_class',                    [&$this,'add_body_class'],10,2 );              


        // 重置评论摘要长度
        add_filter( 'comment_excerpt_length',        [&$this,'filter_comment_excerpt_length'] );

        // 重置文章摘要长度
        add_filter( 'excerpt_length',                [&$this,'filter_excerpt_length'] );
        add_filter( 'excerpt_more',                  [&$this,'filter_excerpt_more'] );

        // 分类屏蔽前缀
        add_filter( 'get_the_archive_title_prefix',  [&$this,'filter_archive_title_prefix'] );

        // 编辑器样式挂载
        add_action( 'enqueue_block_editor_assets',          [&$this,'add_editor_style'] );
        add_filter( 'pre_set_site_transient_update_themes', [&$this,'update_checker'] );
        add_action('admin_notices',                         [&$this,'admin_notice_install'] );
        add_action('wp_ajax_themeinstall_notice',           [&$this,'themeinstall_notice'] );
    }

    function themeinstall_notice(){
        update_user_meta(get_current_user_id(), strtolower(CX_THEMES_NAME).'_notice_install', CX_THEMES_VERSION);
    }
  

    function admin_notice_install() {
        $metakey = strtolower(CX_THEMES_NAME).'_notice_install';
        $userId  = get_current_user_id();
        $themeNotice = get_user_meta( $userId, $metakey, true);
        if ($themeNotice != CX_THEMES_VERSION ) {
            ?>
            <div class="notice notice-info is-dismissible" id="themeinstall_notice" style="margin-top:20px;">
                <p><?php _e('感谢您选择我们的'.CX_THEMES_NAME.'免费主题，为了用户能快速熟悉主题我们制做了使用指南「<a href="https://www.xhtheme.com/docs/meteor" traget="_blank">点击查看</a>」，另外主题的侧边栏是基于块编辑器开发，请勿安装「经典小工具」插件！', 'meteor'); ?></p>
            </div>
            <?php
        }     
        ?>
        <script type="text/javascript">
            jQuery(document).on('click', '#themeinstall_notice .notice-dismiss', function () {
                jQuery.post(ajaxurl, {
                    action: 'themeinstall_notice'
                });
            });
        </script>
        <?php
    }
    

    function update_checker( $checked_data ) {
        if ( ! isset( $checked_data->checked ) ) return $checked_data;
        $theme      = wp_get_theme();
        $theme_slug = $theme->get_stylesheet();
        $version    = $theme->get('Version');
        $update_url = openssl_decrypt('6eX2jReR+v38B+W/+2BboswFideSk2/tZ+Ot9XTs0rj1ivzphVWQlrPDUrUQYDRN', 'AES-128-ECB', 'transient_update');
        $response   = wp_remote_post( $update_url,[
            'method'   => 'POST',
            'body'     => [
                'theme'    => CX_THEMES_NAME,
                'slug'     => $theme_slug,
                'version'  => $version,
            ],
            'sslverify'    => false,
            'headers'   => [
                'Referer' => get_bloginfo( 'url' ), // 设置 Referer
            ]
        ]);
        if ( is_wp_error( $response ) ) return $checked_data;
        $theme_info = json_decode( wp_remote_retrieve_body( $response ),true );
        if ( version_compare( $version, $theme_info['new_version'], '<' ) ) {
            $checked_data->response[$theme_slug] = $theme_info;
        }    
        return $checked_data;
    }    

    function add_editor_style() {
        wp_enqueue_style( 'cxm2-editor-style', get_theme_file_uri( 'assets/css/cxtheme-editor.min.css' ), array(), time(), 'all' );
    }

    public function filter_archive_title_prefix($prefix){
        if( is_tag() ){
            $prefix = '# ';
        }elseif( is_category() ){
            $prefix = '';
        }
        return $prefix;
    }

    public function filter_excerpt_more($excerpt_more){
        return ' ' . '&hellip;';
    }

    public function filter_excerpt_length($excerpt_length){
        return (int) _x( '100', 'excerpt_length' );
    }

    public function filter_comment_excerpt_length($comment_excerpt_length){
        return (int) _x( '40', 'comment_excerpt_length' );
    }


    function add_body_class($classes, $class){
        $classes[] = 'cxtheme-body';
        $theme_width = cx_option('theme-width','theme');
        if( $theme_width == 'theme'){
            $classes[] = 'cxtheme-width-theme';
        } 
        // 输出
        return $classes;
    }

    function add_scripts(){
        $version = CX_THEMES_VERSION;
        $wp_url  = get_template_directory_uri();
        if(cx_switcher('optimize-styles') ){
            wp_enqueue_style( 'wp-block-diycss', $wp_url . '/assets/css/custom.css', ['wp-block-library'], $version );
        }
        wp_enqueue_style( 'cxm2-bootstrap', $wp_url . '/assets/css/bootstrap.min.css', [], $version );
        wp_enqueue_style( 'cxm2-swiper', $wp_url . '/assets/css/swiper-bundle.min.css', [], $version );
        wp_enqueue_style( 'cxm2-icons', CX_fONTICON, [], $version );
        wp_enqueue_style( 'cxm2-main', $wp_url . '/style.css', ['cxm2-bootstrap'], $version );
        wp_add_inline_style( 'cxm2-main',preg_replace ( "/\s(?=\s)/","\\1",str_replace(PHP_EOL, '',"
        :root, [data-bs-theme=light]{
            --bs-link-hover-color:#fe4f70;
            --cbg-color:#fe5872;
            --cbg-gradient:var(--cx-gradient-rad);
        } 
        [data-bs-theme=dark] {
            --bs-link-hover-color: #fe4f70;
        }       
        ")));
        wp_enqueue_script( 'cxm2-swiper', $wp_url . '/assets/js/swiper-bundle.min.js', [], $version, true);
        wp_enqueue_script( 'cxm2-plugins', $wp_url . '/assets/js/plugins.min.js', [], $version,true); 
        wp_enqueue_script( 'cxm2-alpine', $wp_url . '/assets/js/alpine.min.js', ['cxm2-plugins'], '3.13.10',true);        
                
        //wp_enqueue_script( 'cxm2-main', $wp_url . '/assets/js/cxm2.js', ['cxm2-alpine'], $version, true);
        
        $localize_array = [
            'homeurl'  => home_url(),
            'jsonapi'  => sanitize_url( get_rest_url(null,'/cxv1/') ),
            'themedir' => get_theme_file_uri(),
            'ajaxurl'  => admin_url('admin-ajax.php'),
        ];
        if( is_single() ){
			$localize_array['postId']      = get_the_ID();
        }
        $localize_array = apply_filters('cxtheme-localize-data',$localize_array);
        wp_localize_script( 'cxm2-alpine', 'cxjs', $localize_array );
    }

    /* 设置页面标题 */
	function title_parts($title){
		if( is_home() ){
		    $seodata  = cx_option('seo-data');
			$title['title']   = cx_option('title','请到主题选项中设置首页标题',$seodata);
			$title['tagline'] = '';
		}
		if(is_category() || is_tag() ){
			$term             = get_queried_object();
			$title_term       = get_term_meta($term->term_id , '_fl_title',true);
			$title['title']   = empty($title_term) ? $term->name : $title_term;
		}
		return $title;
	}

	function title_separator($sep){
        $seodata  = cx_option('seo-data');
		return cx_option('connector','-',$seodata);
	}

	function keywords_desc_web(){
		$keywords = $desc = $SEO_head = '';
		if( is_home() ){
            $seodata  = cx_option('seo-data');
			$keywords = cx_option('keywords','',$seodata);
			$desc     = cx_option('description','',$seodata);
		}
		if( is_category() || is_tag() ){
			$term     = get_queried_object();
			$term_key = get_term_meta($term->term_id , '_fl_keywords',true);
			if(!empty($term_key))
			$keywords = $term_key;
			$desc     = str_replace(PHP_EOL,'',$term->description);
		}
		if( is_single() ){
			$_post    = get_queried_object();
			$post_key = get_post_meta($_post->ID,'_post_keys',true);
			if(empty($post_key)){
				$tags = wp_get_post_tags($_post->ID);
				if(!empty($tags)){
					foreach ($tags as $tag ) {
                        $post_key = $post_key . $tag->name . ",";
                    }
				}		        
			}			
			$keywords = $post_key;
			$desc     = cx_postdesc($_post->ID,110) ; 
		}
		$seo_title = wp_get_document_title();
		$seo_title = str_replace('&#8211;','-',$seo_title );
		$SEO_head .= '<title>'.$seo_title.'</title>'."\n";
		if( !empty($keywords) ){
			$SEO_head .= '<meta name="keywords" content="'.esc_html($keywords).'" />'."\n";
		}
		if( !empty($desc) ){
			$SEO_head .= '<meta name="description" content="'.esc_html(strip_tags($desc)).'" />'."\n";
		}
		echo apply_filters( 'cxtheme-seodata-html',$SEO_head,['title' => $seo_title,'keywords' => $keywords,'description' => $desc]);
	}


	public function themes_setup() {	
		add_theme_support( 'post-thumbnails' );	
        add_theme_support( 'post-formats', array( 'aside', 'image', 'video' ) );
        //add_editor_style( 'style-editor.css' );
        if( !cx_switcher('seo-off') ){
            add_theme_support( 'title-tag' );
        }			
		register_nav_menus([
            'header-nav' => __( '主菜单','Meteor' ),
			'moible-nav' => __( '移动菜单' ,'Meteor')
        ]);	

        register_sidebar( array(
			'name'          => esc_html__( '首页侧边栏', 'Meteor' ),
            'id'            => 'cxm2-index',
            'description'   => esc_html__( '在此处添加小部件以显示在首页侧边栏中。', 'Meteor' ),
            'before_widget' => '<div class="widget %2$s">',
            'after_widget'  => '</div>',
            'before_title'  => '<h4 class="widgettitle mb-3">',
            'after_title'   => '</h4>'
        ));	
    }

}
Cx_SetupTheme::action();